/*! 
* 
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this file,
* You can obtain one at http://mozilla.org/MPL/2.0/.
* 
* Copyright(c) 2012 Apogee Imaging Systems, Inc. 
* 
* \class IAltaSerialPortIo 
* \brief virtual interface for the AltaU/E serial port access 
* 
*/ 

#include "IAltaSerialPortIo.h" 


//////////////////////////// 
// DTOR 
IAltaSerialPortIo::~IAltaSerialPortIo() 
{ 

}
