// GUID_USB.h     
//
// Copyright (c) 2003 Apogee Instruments Inc.
//
// Definition for GUID_USBWDM_DEVICE_INTERFACE_CLASS
//
// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this file,
// You can obtain one at http://mozilla.org/MPL/2.0/.
//

#include <initguid.h>

#ifdef DEFINE_GUID

// {09E3D9A1-8E70-496c-BC1D-FC8A43CEF6B5}
DEFINE_GUID(GUID_USBWDM_DEVICE_INTERFACE_CLASS, 
0x9e3d9a1, 0x8e70, 0x496c, 0xbc, 0x1d, 0xfc, 0x8a, 0x43, 0xce, 0xf6, 0xb5);

#endif
